const { Events, ChannelType, PermissionFlagsBits, ActionRowBuilder, ButtonBuilder, ButtonStyle, EmbedBuilder } = require("discord.js");
const randomstring = require('randomstring')
const Event = require("../../structures/Event");

module.exports = class extends Event {
    constructor(client) {
        super(client, {
            name: Events.InteractionCreate
        });
    }

    run = async (interaction) => {
        const client = this.client;
        if (!interaction.isStringSelectMenu()) return;

        if (interaction.customId == 'suporte.menu') {
            try {
                let random = randomstring.generate({
                    length: 8,
                    charset: 'alphanumeric'
                });

                let values = interaction.values[0];

                if (values) {

                    let parent = await client.config.Guild.Parents[values]

                    await interaction.guild.channels.create({
                        name: `🎫・${values}-${random}`,
                        type: ChannelType.GuildText,

                        permissionOverwrites: [
                            {
                                id: interaction.guild.roles.everyone,
                                deny: [PermissionFlagsBits.ViewChannel]
                            },
                            {
                                id: interaction.user.id,
                                allow: [PermissionFlagsBits.ViewChannel, PermissionFlagsBits.SendMessages, PermissionFlagsBits.ReadMessageHistory]
                            },
                            {
                                id: client.config.Guild.Roles.Equipe,
                                allow: [PermissionFlagsBits.ViewChannel, PermissionFlagsBits.SendMessages, PermissionFlagsBits.ReadMessageHistory]
                            }
                        ],

                        parent: parent
                    }).then(async (channel) => {
                        await interaction.reply({
                            content: `🚀 • Você abriu com sucesso um ticket. Seu ticket: ${channel}`,
                            ephemeral: true
                        });

                        await channel.setTopic(`<:users:1276883311460880490>・ID do membro: **${interaction.user.id}** \n<:channel:1276884856126701691>・ID do Ticket: **${random}** \n<:info_:1276885069792940142>・Ticket assumido por: **Ninguém**`)

                        const serverURL = await interaction.guild.iconURL();
                        const userURL = await interaction.user.displayAvatarURL({ dynamic: true, format: 'png', size: 1024 })
                        const embed = new EmbedBuilder()
                            .setAuthor({
                                name: `PT-SKILLS`,
                                iconURL: `${serverURL}`
                            })
                            .setTitle(`<:ticket:1202703343294742581> • Suporte`)
                            .setDescription(`👋 Olá ${interaction.user},
                            
❤️ Obrigado por abrir ticket na **PT-SKILLS**. Aguarde até que a nossa equipe lhe responda.

<:channel:1276884856126701691> ID do Ticket: **${random}**
📁 Tipo de ticket: **${values}**
<:info_:1276885069792940142> Ticket assumido por: **Ninguém**`)
                            .setColor("Blue")
                            .setThumbnail(userURL)
                            .setFooter({
                                text: `${this.client.config.Copyright}`,
                                iconURL: `${serverURL}`
                            })
                            .setTimestamp();
                        
                        const row = new ActionRowBuilder()
                        .addComponents(
                            new ButtonBuilder()
                            .setCustomId(`ticket.assumir-${channel.id}`)
                            .setDisabled(false)
                            .setEmoji("<:users:1276883311460880490>")
                            .setLabel("Assumir Ticket")
                            .setStyle(ButtonStyle.Secondary),

                            new ButtonBuilder()
                            .setCustomId(`ticket.notificar-${channel.id}`)
                            .setDisabled(false)
                            .setEmoji("<a:notify:1276919603276156993>")
                            .setLabel("Notificar Membro")
                            .setStyle(ButtonStyle.Secondary),

                            new ButtonBuilder()
                            .setCustomId(`ticket.adicionar-${channel.id}`)
                            .setDisabled(false)
                            .setEmoji("<:add_membro:1276885708157751408>")
                            .setLabel("Adicionar Membro")
                            .setStyle(ButtonStyle.Secondary),

                            new ButtonBuilder()
                            .setCustomId(`ticket.remover-${channel.id}`)
                            .setDisabled(false)
                            .setEmoji("<:member:1276885787975090197>")
                            .setLabel("Remover Membro")
                            .setStyle(ButtonStyle.Secondary),
                        )

                        const row2 = new ActionRowBuilder()
                        .addComponents(
                            new ButtonBuilder()
                            .setCustomId(`ticket.voz-${channel.id}`)
                            .setDisabled(false)
                            .setEmoji("<:channel:1276884856126701691>")
                            .setLabel("Criar Canal (Voz)")
                            .setStyle(ButtonStyle.Secondary),

                            new ButtonBuilder()
                            .setCustomId(`ticket.fechar-${channel.id}`)
                            .setDisabled(false)
                            .setEmoji("<:remove:1276885933299339356>")
                            .setLabel("Fechar Ticket")
                            .setStyle(ButtonStyle.Secondary)
                        )

                        let message = await channel.send({
                            embeds: [ embed ],
                            components: [ row, row2 ]
                        });

                        await message.pin();

                        client.tickets.set(`${channel.id}`, { number: `${random}`, row_1: row, row_2: row2, tipo: `${values}`, content: embed.data, author: interaction.user.id });
                    });
                };
            } catch(err) {
                console.log(err)
                return await interaction.reply({
                    content: `<a:wumpus:1276886038564049020> • Ocorreu um erro ao executar a função!`,
                    ephemeral: true
                });
            };
        };
    };
}