const { Events, ChannelType, PermissionFlagsBits, ActionRowBuilder, UserSelectMenuBuilder, EmbedBuilder, ButtonStyle, ButtonBuilder } = require("discord.js");
const fs = require("fs");
const path = require("path");
const Event = require("../../structures/Event");

module.exports = class extends Event {
    constructor(client) {
        super(client, {
            name: Events.InteractionCreate
        });
    }

    run = async (interaction) => {
        const client = this.client;
        if (!interaction.isButton()) return;

        try {
            if (interaction.customId == `ticket.assumir-${interaction.channel.id}`) {
                if (!interaction.member.roles.cache.has(client.config.Guild.Roles.Equipe)) {
                    await interaction.reply({
                        content: `<a:fechar:1276883044900536391>・Apenas staffs podem assumir tickets!`,
                        ephemeral: true
                    });
                };

                let ticket = await client.tickets.get(`${interaction.channel.id}`)

                await interaction.channel.setTopic(`<:users:1276883311460880490>・ID do membro: **${ticket.author}** \n<:channel:1276884856126701691>・ID do Ticket: **${ticket.number}** \n<:info_:1276885069792940142>・Ticket assumido por: **${interaction.user.id}**`);

                let row = ticket.row_1
                let row2 = ticket.row_2
                let data = ticket.content;

                row.components.forEach(async btn => {
                    if (btn.data.custom_id == `ticket.assumir-${interaction.channel.id}`) {
                        btn.data.disabled = true;

                        let desc = await interaction.message.embeds[0].description.replace('Ninguém', interaction.user);

                        data.description = desc;

                        await interaction.message.edit({
                            embeds: [ data ],
                            components: [ row, row2 ]
                        });
                    };
                });

                await interaction.reply({
                    content: `<:right:1276886612797554709>・Você assumiu o ticket com sucesso!`,
                    ephemeral: true
                });
            } else if (interaction.customId == `ticket.notificar-${interaction.channel.id}`) {
                if (!interaction.member.roles.cache.has(client.config.Guild.Roles.Equipe)) {
                    await interaction.reply({
                        content: `<a:fechar:1276883044900536391>・Apenas staffs podem assumir tickets!`,
                        ephemeral: true
                    });
                };

                let userID = await client.tickets.get(`${interaction.channel.id}`);
                let user = await client.users.cache.get(userID.author);

                if (!user) {
                    await interaction.reply({
                        content: `<:info_2:1276886862610567208>・Ocorreu um erro ao encontrar o usuário que abriu o ticket!`,
                        ephemeral: true
                    });
                };

                await user.createDM();
                await user.send({
                    content: `<:folha:1276882731145494652>・${user}, um staff está aguardando a sua resposta no ticket!!\n<:link:1276886982383108218> [Clique aqui](https://discord.com/channels/${interaction.guild.id}/${interaction.channel.id}) para ir ao seu ticket!`
                });
                await user.deleteDM();

                await interaction.reply({
                    content: `<:right:1276886612797554709>・Você notificou o ${user} com sucesso!`,
                    ephemeral: true
                });
            } else if (interaction.customId == `ticket.adicionar-${interaction.channel.id}`) {
                if (!interaction.member.roles.cache.has(client.config.Guild.Roles.Equipe)) {
                    await interaction.reply({
                        content: `<a:fechar:1276883044900536391>・Apenas staffs podem assumir tickets!`,
                        ephemeral: true
                    });
                };

                let row = new ActionRowBuilder()
                .addComponents(
                    new UserSelectMenuBuilder()
                    .setCustomId(`ticket.add.user-${interaction.channel.id}`)
                    .setDisabled(false)
                    .setMaxValues(1)
                    .setMinValues(0)
                    .setPlaceholder("Selecione o usuário.")
                );

                await interaction.reply({
                    components: [ row ],
                    ephemeral: true
                })
            } else if (interaction.customId == `ticket.remover-${interaction.channel.id}`) {
                if (!interaction.member.roles.cache.has(client.config.Guild.Roles.Equipe)) {
                    await interaction.reply({
                        content: `<a:fechar:1276883044900536391>・Apenas staffs podem assumir tickets!`,
                        ephemeral: true
                    });
                };

                let row = new ActionRowBuilder()
                .addComponents(
                    new UserSelectMenuBuilder()
                    .setCustomId(`ticket.remove.user-${interaction.channel.id}`)
                    .setDisabled(false)
                    .setMaxValues(1)
                    .setMinValues(0)
                    .setPlaceholder("Selecione o usuário.")
                );

                await interaction.reply({
                    components: [ row ],
                    ephemeral: true
                })
            } else if (interaction.customId == `ticket.voz-${interaction.channel.id}`) {
                let userID = await client.tickets.get(`${interaction.channel.id}`);
                let user = await client.users.cache.get(userID.author);

                if (!user) {
                    await interaction.reply({
                        content: `<:info_2:1276886862610567208>・Ocorreu um erro ao encontrar o usuário que abriu o ticket!`,
                        ephemeral: true
                    });
                };

                await interaction.guild.channels.create({
                    name: `
                    Suporte Voz de ${user.username}`,
                    type: ChannelType.GuildVoice,

                    permissionOverwrites: [
                        {
                            id: interaction.guild.roles.everyone,
                            deny: [PermissionFlagsBits.ViewChannel]
                        },
                        {
                            id: interaction.user.id,
                            allow: [PermissionFlagsBits.Connect, PermissionFlagsBits.Stream, PermissionFlagsBits.Speak]
                        },
                        {
                            id: client.config.Guild.Roles.Equipe,
                            allow: [PermissionFlagsBits.Connect, PermissionFlagsBits.Stream, PermissionFlagsBits.Speak]
                        }
                    ],

                    parent: interaction.channel.parent
                });

                await interaction.reply({
                    content: `<:voice:1276887276978438184>・Canal de Voz criado com sucesso!`,
                    ephemeral: true
                });
            } else if (interaction.customId == `ticket.fechar-${interaction.channel.id}`) {
                let embedOld = await interaction.message.embeds[0].description;
                const regex = /Ticket assumido por: (\S+)/;

                let assumiu;
                let userID = await client.tickets.get(`${interaction.channel.id}`);
                let user = await client.users.cache.get(userID.author);

                if (!user) {
                    await interaction.reply({
                        content: `<:info_2:1276886862610567208>・Ocorreu um erro ao encontrar o usuário que abriu o ticket!`,
                        ephemeral: true
                    });
                }

                const match = embedOld.match(regex);

                if (match) {
                    assumiu = match[1]
                } else {
                    assumiu = `Desconhecido`
                }

                const serverURL = await interaction.guild.iconURL();
                const embed = new EmbedBuilder()
                .setAuthor({
                    name: `PT-SKILLS`,
                    iconURL: `${serverURL}`
                })
                .setTitle(`📃・Logs Ticket`)
                .setDescription(`<:ticket_fecahdo:1276919274472079413>・Ticket fechado por: **${interaction.user}**
<:ticket_aberto:1276919395817623665>・Ticket aberto por: **${user}**
<:info_:1276885069792940142>・Ticket assumido por: **${assumiu}**
<:channel:1276884856126701691>・ID do Ticket: **${userID.number}**`)
                .setColor("#4b02cc")
                .setThumbnail(`${serverURL}`)
                .setFooter({
                    text: `${this.client.config.Copyright}`,
                    iconURL: `${serverURL}`
                })
                .setTimestamp();

                let response = await interaction.guild.channels.cache.get(client.config.Guild.Channels.Logs).send({
                    embeds: [ embed ]
                });

                let row = new ActionRowBuilder()
                .addComponents(
                    new ButtonBuilder()
                    .setCustomId("ticket-" + userID.number)
                    .setDisabled(false)
                    .setEmoji("<:info_2:1276886862610567208>")
                    .setLabel("Transcript")
                    .setStyle(ButtonStyle.Primary)
                )

                await response.edit({
                    components: [ row ]
                });

                const transcript_dir = path.join("transcript");

                if (!fs.existsSync(transcript_dir)) {
                    fs.mkdirSync(transcript_dir, { recursive: true });
                };

                try {
                    let text = "";

                    await interaction.channel.messages.fetch({}).then(async messages => {
                        text = messages.map(msg => `${msg.author.tag}: ${msg.content}`).join('\n');
                    });

                    const filePath = path.join("transcript", `transcript-${userID.number}.txt`);

                    let file = await fs.writeFileSync(filePath, text);

                    let channel = await interaction.guild.channels.cache.find(chann => chann.name.includes(user.username));

                    if (channel) {
                        await channel.delete(`O ticket acabou`);
                    };
    
                    await interaction.channel.delete(`O ticket acabou`);
                } catch(err) {
                    console.log(`[Transcript] -> Erro ao escrever/criar o arquivo.\n${err}`)
                }
            }
        } catch(err) {
            console.log(err)
            return await interaction.reply({
                content: `<a:wumpus:1276886038564049020>・Ocorreu um erro ao executar a função!`,
                ephemeral: true
            });
        };
    };
}