const { Client, LimitedCollection, Collection, GatewayIntentBits } = require("discord.js");
const { readdirSync } = require("fs")
const { join } = require("path")
const app = require("./Express");
const config = require("../config/config");

module.exports = class extends Client {
    constructor(options) {
        super({
            ...options,
            intents: [
                GatewayIntentBits.AutoModerationConfiguration,
                GatewayIntentBits.AutoModerationExecution,
                GatewayIntentBits.DirectMessageReactions,
                GatewayIntentBits.DirectMessageTyping,
                GatewayIntentBits.DirectMessages,
                GatewayIntentBits.GuildEmojisAndStickers,
                GatewayIntentBits.GuildIntegrations,
                GatewayIntentBits.GuildInvites,
                GatewayIntentBits.GuildMembers,
                GatewayIntentBits.GuildMessageReactions,
                GatewayIntentBits.GuildMessageTyping,
                GatewayIntentBits.GuildMessages,
                GatewayIntentBits.GuildModeration,
                GatewayIntentBits.GuildPresences,
                GatewayIntentBits.GuildScheduledEvents,
                GatewayIntentBits.GuildVoiceStates,
                GatewayIntentBits.GuildWebhooks,
                GatewayIntentBits.Guilds,
                GatewayIntentBits.MessageContent,
            ],

            makeCache: manager => {
                if (manager.name === 'MessageManager') return new LimitedCollection({ maxSize: 0 });
                return new Collection();
            },
            allowedMentions: { 
                parse: [
                    'users', 'roles'
                ], 
                repliedUser: true 
            }
        })

        this.commands = [];
        this.loadCommands();
        this.loadEvents();
        this.config = config;
        this.app = app;
        this.tickets = new Map();
    }

    async SlashCommands() {
        await this.application.commands.set(this.commands);
        console.log("[SlashCommands] -> Slash Commands Carregados!");
    }

    loadCommands(path = 'commands') {
        const categories = readdirSync(path)

        for (const category of categories) {
            const commands = readdirSync(`${path}/${category}`)

            for (const command of commands) {
                const commandClass = require(join(process.cwd(), `${path}/${category}/${command}`))
                const cmd = new (commandClass)(this)

                this.commands.push(cmd)
                console.log(`[Command] -> Comando --> ${cmd.name} <-- carregado em ${category}`)
            }
        }
    }

    loadEvents(path = 'src/events') {
        const categorias = readdirSync(path);

        for ( const categoria of categorias ) {
            const events = readdirSync(`${path}/${categoria}`);

            for ( const event of events ) {
                const eventClass = require(join(process.cwd(), `${path}/${categoria}/${event}`));

                const evt = new (eventClass)(this);

                this.on(evt.name, evt.run);
                console.log(`[Events] -> Evento --> ${evt.name} <-- carregado em ${categoria}`)
            };
        };
    };

    /*async connectToDatabase() {
        await mongoose.connect(process.env.MONGO_URL, {
            
        }).then(() => {
            console.log(`[Database] -> Database Conectada!`)
        }).catch(err => {
            console.log(`[Database] -> Database não Conectada!`)
        });
        
        this.db = mongoose.connect;
    };*/
}